@@ -25,46 +25,46 @@ module Ide.Plugin.Eval.CodeLens (
 
 import           Control.Applicative                          (Alternative ((<|>)))
 import           Control.Arrow                                (second, (>>>))
-import           Control.Exception                            (try)
+import           Control.Exception                            (try, bracket_)
 import qualified Control.Exception                            as E
 import           Control.Lens                                 (_1, _3, ix, (%~),
                                                                (<&>), (^.))
-import           Control.Monad                                (guard,
-                                                               void, when)
+import           Control.Monad                                (guard, void,
+                                                               when)
 import           Control.Monad.IO.Class                       (MonadIO (liftIO))
-import           Control.Monad.Trans.Except                   (ExceptT (..))
+import           Control.Monad.Trans.Except                   (ExceptT (..),
+                                                               runExceptT)
 import           Data.Aeson                                   (toJSON)
 import           Data.Char                                    (isSpace)
 import           Data.Foldable                                (toList)
-import qualified Data.HashMap.Strict                          as HashMap
 import           Data.List                                    (dropWhileEnd,
                                                                find,
                                                                intercalate,
                                                                intersperse)
+import qualified Data.Map                                     as Map
 import           Data.Maybe                                   (catMaybes)
 import           Data.String                                  (IsString)
 import           Data.Text                                    (Text)
 import qualified Data.Text                                    as T
 import           Data.Typeable                                (Typeable)
-import Development.IDE.Core.RuleTypes
-    ( NeedsCompilation(NeedsCompilation),
-      LinkableResult(linkableHomeMod),
-      tmrTypechecked,
-      TypeCheck(..))
-import Development.IDE.Core.Rules ( runAction, IdeState )
-import Development.IDE.Core.Shake
-    ( useWithStale_,
-      use_,
-      uses_ )
-import Development.IDE.GHC.Util
-    ( printOutputable, evalGhcEnv, modifyDynFlags )
-import Development.IDE.Types.Location
-    ( toNormalizedFilePath', uriToFilePath' )
+import           Development.IDE.Core.Rules                   (IdeState,
+                                                               runAction)
+import           Development.IDE.Core.RuleTypes               (LinkableResult (linkableHomeMod),
+                                                               NeedsCompilation (NeedsCompilation),
+                                                               TypeCheck (..),
+                                                               tmrTypechecked)
+import           Development.IDE.Core.Shake                   (useWithStale_, useNoFile_,
+                                                               use_, uses_)
 import           Development.IDE.GHC.Compat                   hiding (typeKind,
                                                                unitState)
 import           Development.IDE.GHC.Compat.Util              (GhcException,
                                                                OverridingBool (..))
-import           Development.IDE.Import.DependencyInformation (reachableModules)
+import           Development.IDE.GHC.Util                     (evalGhcEnv,
+                                                               modifyDynFlags,
+                                                               printOutputable)
+import           Development.IDE.Import.DependencyInformation (transitiveDeps, transitiveModuleDeps)
+import           Development.IDE.Types.Location               (toNormalizedFilePath',
+                                                               uriToFilePath')
 import           GHC                                          (ClsInst,
                                                                ExecOptions (execLineNumber, execSourceFile),
                                                                FamInst,
@@ -75,33 +75,36 @@ import           GHC                                          (ClsInst,
                                                                exprType,
                                                                getInfo,
                                                                getInteractiveDynFlags,
-                                                               isImport, isStmt, parseName,
+                                                               isImport, isStmt,
+                                                               parseName,
                                                                pprFamInst,
                                                                pprInstance,
                                                                typeKind)
 
 
-import Development.IDE.Core.RuleTypes
-    ( ModSummaryResult(msrModSummary),
-      GetModSummary(GetModSummary),
-      GhcSessionDeps(GhcSessionDeps),
-      GetDependencyInformation(GetDependencyInformation),
-      GetLinkable(GetLinkable) )
-import Development.IDE.Core.Shake ( VFSModified(VFSUnmodified) )
-import Development.IDE.Types.HscEnvEq ( HscEnvEq(hscEnv) )
-import qualified Development.IDE.GHC.Compat.Core as Compat
-    ( InteractiveImport(IIModule) )
-import qualified Development.IDE.GHC.Compat.Core as SrcLoc
-    ( unLoc, HasSrcSpan(getLoc) )
+import           Development.IDE.Core.RuleTypes               (GetModuleGraph (GetModuleGraph),
+                                                               GetLinkable (GetLinkable),
+                                                               GetModSummary (GetModSummary),
+                                                               GhcSessionDeps (GhcSessionDeps),
+                                                               ModSummaryResult (msrModSummary))
+import           Development.IDE.Core.Shake                   (VFSModified (VFSUnmodified))
+import qualified Development.IDE.GHC.Compat.Core              as Compat (InteractiveImport (IIModule))
+import qualified Development.IDE.GHC.Compat.Core              as SrcLoc (HasSrcSpan (getLoc),
+                                                                         unLoc)
+import           Development.IDE.Types.HscEnvEq               (HscEnvEq (hscEnv))
 #if MIN_VERSION_ghc(9,2,0)
 #endif
 import qualified GHC.LanguageExtensions.Type                  as LangExt (Extension (..))
 
 import           Development.IDE.Core.FileStore               (setSomethingModified)
+import           Development.IDE.Core.PluginUtils
 import           Development.IDE.Types.Shake                  (toKey)
 #if MIN_VERSION_ghc(9,0,0)
 import           GHC.Types.SrcLoc                             (UnhelpfulSpanReason (UnhelpfulInteractive))
 #endif
+import           Ide.Plugin.Error                             (PluginError (PluginInternalError),
+                                                               handleMaybe,
+                                                               handleMaybeM)
 import           Ide.Plugin.Eval.Code                         (Statement,
                                                                asStatements,
                                                                myExecStmt,
@@ -117,39 +120,35 @@ import           Ide.Plugin.Eval.GHC                          (addImport,
                                                                showDynFlags)
 import           Ide.Plugin.Eval.Parse.Comments               (commentsToSections)
 import           Ide.Plugin.Eval.Parse.Option                 (parseSetFlags)
-import           Ide.Plugin.Eval.Rules                        (queueForEvaluation)
+import           Ide.Plugin.Eval.Rules                        (queueForEvaluation, unqueueForEvaluation)
 import           Ide.Plugin.Eval.Types
 import           Ide.Plugin.Eval.Util                         (gStrictTry,
                                                                isLiterate,
                                                                logWith,
                                                                response', timed)
-import           Ide.PluginUtils                              (handleMaybe,
-                                                               handleMaybeM,
-                                                               pluginResponse)
 import           Ide.Types
+import qualified Language.LSP.Protocol.Lens                   as L
+import           Language.LSP.Protocol.Message
+import           Language.LSP.Protocol.Types
 import           Language.LSP.Server
-import           Language.LSP.Types                           hiding
-                                                              (SemanticTokenAbsolute (length, line),
-                                                               SemanticTokenRelative (length))
-import           Language.LSP.Types.Lens                      (end, line)
 import           Language.LSP.VFS                             (virtualFileText)
 
 {- | Code Lens provider
  NOTE: Invoked every time the document is modified, not just when the document is saved.
 -}
-codeLens :: PluginMethodHandler IdeState TextDocumentCodeLens
+codeLens :: PluginMethodHandler IdeState Method_TextDocumentCodeLens
 codeLens st plId CodeLensParams{_textDocument} =
     let dbg = logWith st
         perf = timed dbg
      in perf "codeLens" $
-            pluginResponse $ do
+            do
                 let TextDocumentIdentifier uri = _textDocument
-                fp <- handleMaybe "uri" $ uriToFilePath' uri
+                fp <- uriToFilePathE uri
                 let nfp = toNormalizedFilePath' fp
                     isLHS = isLiterate fp
                 dbg "fp" fp
-                (comments, _) <- liftIO $
-                    runAction "eval.GetParsedModuleWithComments" st $ useWithStale_ GetEvalComments nfp
+                (comments, _) <-
+                    runActionE "eval.GetParsedModuleWithComments" st $ useWithStaleE GetEvalComments nfp
                 -- dbg "excluded comments" $ show $  DL.toList $
                 --     foldMap (\(L a b) ->
                 --         case b of
@@ -171,7 +170,7 @@ codeLens st plId CodeLensParams{_textDocument} =
                               args = EvalParams (setupSections ++ [section]) _textDocument ident
                               cmd' =
                                 (cmd :: Command)
-                                    { _arguments = Just (List [toJSON args])
+                                    { _arguments = Just [toJSON args]
                                     , _title =
                                         if trivial resultRange
                                             then "Evaluate..."
@@ -192,7 +191,7 @@ codeLens st plId CodeLensParams{_textDocument} =
                             , "lenses."
                             ]
 
-                return $ List lenses
+                return $ InL lenses
   where
     trivial (Range p p') = p == p'
 
@@ -208,22 +207,22 @@ runEvalCmd :: PluginId -> CommandFunction IdeState EvalParams
 runEvalCmd plId st EvalParams{..} =
     let dbg = logWith st
         perf = timed dbg
-        cmd :: ExceptT String (LspM Config) WorkspaceEdit
+        cmd :: ExceptT PluginError (LspM Config) WorkspaceEdit
         cmd = do
             let tests = map (\(a,_,b) -> (a,b)) $ testsBySection sections
 
             let TextDocumentIdentifier{_uri} = module_
-            fp <- handleMaybe "uri" $ uriToFilePath' _uri
+            fp <- uriToFilePathE _uri
             let nfp = toNormalizedFilePath' fp
             mdlText <- moduleText _uri
 
             -- enable codegen for the module which we need to evaluate.
-            liftIO $ queueForEvaluation st nfp
-            liftIO $ setSomethingModified VFSUnmodified st [toKey NeedsCompilation nfp] "Eval"
-            -- Setup a session with linkables for all dependencies and GHCi specific options
-            final_hscEnv <- liftIO $ initialiseSessionForEval
-                                      (needsQuickCheck tests)
-                                      st nfp
+            final_hscEnv <- liftIO $ bracket_
+              (do queueForEvaluation st nfp
+                  setSomethingModified VFSUnmodified st [toKey IsEvaluating nfp] "Eval")
+              (do unqueueForEvaluation st nfp
+                  setSomethingModified VFSUnmodified st [toKey IsEvaluating nfp] "Eval")
+              (initialiseSessionForEval (needsQuickCheck tests) st nfp)
 
             evalCfg <- liftIO $ runAction "eval: config" st $ getEvalConfig plId
 
@@ -234,13 +233,13 @@ runEvalCmd plId st EvalParams{..} =
                         evalGhcEnv final_hscEnv $ do
                             runTests evalCfg (st, fp) tests
 
-            let workspaceEditsMap = HashMap.fromList [(_uri, List $ addFinalReturn mdlText edits)]
+            let workspaceEditsMap = Map.fromList [(_uri, addFinalReturn mdlText edits)]
             let workspaceEdits = WorkspaceEdit (Just workspaceEditsMap) Nothing Nothing
 
             return workspaceEdits
-     in perf "evalCmd" $
+     in perf "evalCmd" $ ExceptT $
             withIndefiniteProgress "Evaluating" Cancellable $
-                response' cmd
+                runExceptT $ response' cmd
 
 -- | Create an HscEnv which is suitable for performing interactive evaluation.
 -- All necessary home modules will have linkables and the current module will
@@ -254,8 +253,8 @@ initialiseSessionForEval needs_quickcheck st nfp = do
     ms <- msrModSummary <$> use_ GetModSummary nfp
     deps_hsc <- hscEnv <$> use_ GhcSessionDeps nfp
 
-    linkables_needed <- reachableModules <$> use_ GetDependencyInformation nfp
-    linkables <- uses_ GetLinkable linkables_needed
+    linkables_needed <- transitiveDeps <$> useNoFile_ GetModuleGraph <*> pure nfp
+    linkables <- uses_ GetLinkable (nfp : maybe [] transitiveModuleDeps linkables_needed)
     -- We unset the global rdr env in mi_globals when we generate interfaces
     -- See Note [Clearing mi_globals after generating an iface]
     -- However, the eval plugin (setContext specifically) requires the rdr_env
@@ -272,7 +271,7 @@ initialiseSessionForEval needs_quickcheck st nfp = do
     return (ms, linkable_hsc)
   -- Bit awkward we need to use evalGhcEnv here but setContext requires to run
   -- in the Ghc monad
-  env2 <- evalGhcEnv env1 $ do
+  env2 <- liftIO $ evalGhcEnv env1 $ do
             setContext [Compat.IIModule (moduleName (ms_mod ms))]
             let df = flip xopt_set    LangExt.ExtendedDefaultRules
                    . flip xopt_unset  LangExt.MonomorphismRestriction
@@ -300,9 +299,9 @@ finalReturn txt =
         p = Position l c
      in TextEdit (Range p p) "\n"
 
-moduleText :: (IsString e, MonadLsp c m) => Uri -> ExceptT e m Text
+moduleText :: MonadLsp c m => Uri -> ExceptT PluginError m Text
 moduleText uri =
-    handleMaybeM "mdlText" $
+    handleMaybeM (PluginInternalError "mdlText") $
       (virtualFileText <$>)
           <$> getVirtualFile
               (toNormalizedUri uri)
@@ -355,18 +354,18 @@ runTests EvalConfig{..} e@(_st, _) tests = do
 asEdit :: Format -> Test -> [Text] -> TextEdit
 asEdit (MultiLine commRange) test resultLines
     -- A test in a block comment, ending with @-\}@ without newline in-between.
-    | testRange test ^. end.line == commRange ^. end . line
+    | testRange test ^. L.end . L.line == commRange ^. L.end . L.line
     =
     TextEdit
         (Range
-            (testRange test ^. end)
-            (resultRange test ^. end)
+            (testRange test ^. L.end)
+            (resultRange test ^. L.end)
         )
         ("\n" <> T.unlines (resultLines <> ["-}"]))
 asEdit _ test resultLines =
     TextEdit (resultRange test) (T.unlines resultLines)
 
-{-
+{- |
 The result of evaluating a test line can be:
 * a value
 * nothing
